Module: Xolo::Admin::TitleEditor
- Defined in:
- lib/xolo/admin/title_editor.rb
Overview
Methods that process the xadm commands and their options
Constant Summary collapse
- TITLE_EDITOR_ROUTE_BASE =
Constants
'/title-editor'- TITLES_ROUTE =
Xolo server route to the list of titles
"#{TITLE_EDITOR_ROUTE_BASE}/titles"
Class Method Summary collapse
-
.extended(extender) ⇒ Object
when this module is extended.
-
.included(includer) ⇒ Object
when this module is included.
Instance Method Summary collapse
-
#ted_titles ⇒ Array<String>
Perhaps not needed for anything, but used for initial connection testing.
Class Method Details
.extended(extender) ⇒ Object
when this module is extended
38 39 40 |
# File 'lib/xolo/admin/title_editor.rb', line 38 def self.extended(extender) Xolo.verbose_extend extender, self end |
.included(includer) ⇒ Object
when this module is included
33 34 35 |
# File 'lib/xolo/admin/title_editor.rb', line 33 def self.included(includer) Xolo.verbose_include includer, self end |
Instance Method Details
#ted_titles ⇒ Array<String>
Perhaps not needed for anything, but used for initial connection testing
49 50 51 |
# File 'lib/xolo/admin/title_editor.rb', line 49 def ted_titles server_cnx.get(TITLES_ROUTE).body end |